@@ -152,8 +152,11 @@ statement : create
   | select
   | copy
   | readin_symbol
+  | commit
   | <error>
 
+commit : /commit/i ';'
+
 connect : /^\s*\\\connect.*\n/
 
 set : /set/i /[^;]*/ ';'
