@@ -1,5 +1,4 @@
-package # hide from pause
-  SQL::Translator::ProducerUtils;
+package SQL::Translator::ProducerUtils;
 
 use Moo;
 use Sub::Quote 'quote_sub';
